<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\Repositories;
use Ems\Util\Utils;

use Ems\Services\ParamOrgService;
use Ems\Services\ApuracaoService;
use Ems\Repository\ResumoRepository;
use Ems\Util\ExpectedException;
use Util\NovoCodigo;

class ResumoService {
    /** @param EntityManager */
    protected $entityManager;

    /** @param Environment */
    protected $environment;

    /** @param ParamOrgService */
    protected $paramOrgService;

    /** @param ParamOrgService */
    protected $apuracaoService;

    /** @param ResumoRepository */
    protected $resumoRepository;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        ResumoRepository $resumoRepository,
        ParamOrgService $paramOrgService,
        ApuracaoService $apuracaoService
    ) {
        $this->entityManager        = $entityManager;
        $this->environment          = $environment;
        $this->resumoRepository     = $resumoRepository;
        $this->paramOrgService      = $paramOrgService;
        $this->apuracaoService      = $apuracaoService;
    }

    public function getSituacoesFreq() {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtCompetencia,
        ];

        $situacoes = $this->resumoRepository->getSituacoesFreq($params);

        return $situacoes;
    }

    public function saveResumoFreq(
        int $nrVinculo,
        int $nrPeriodo,
        string $cdOperador,
        string $date,
        array $rows
    ) {
        $this->saveEditRows(
            $nrVinculo,
            $nrPeriodo,
            $cdOperador,
            $date,
            $rows
        );

        try {
            $this->apuracaoService->apurarDiasPosteriores(
                $nrVinculo,
                $date
            );
        } catch (\Exception $e) {
            throw new ExpectedException($e->getMessage(), $e->getCode(), $e);
        }
    }

    public function saveEditRows(
        int $nrVinculo,
        int $nrPeriodo,
        string $cdOperador,
        string $date,
        array $rows
    ) {
        foreach ($rows as $row) {
            if ($row['NRRESUMOSITUFREQ'] !== null) {
                $this->editRowResumoFreq(
                    $cdOperador,
                    $row
                );
            } else {
                $this->saveRowResumoFreq(
                    $nrVinculo,
                    $nrPeriodo,
                    $cdOperador,
                    $date,
                    $row
                );
            }
        }
    }

    public function saveRowResumoFreq(
        int $nrVinculo,
        int $nrPeriodo,
        string $cdOperador,
        string $date,
        array $row
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrSituFreq = $row['NRSITUFREQUENCIA'] ?? null;
        $qtReferencia = $row['QTREFAPURSITUFREQ'] ?? null;

        $nrResumoFreq = NovoCodigo::geraCodigo(
            $nrOrg, 'PTO_RESUMOSITUFREQ', 1,
            10, true
        );

        if (!isset($nrSituFreq) || !isset($qtReferencia)) {
            return;
        }

        $params = [
            'NRORG'             => $nrOrg,
            'NRVINCULOM'        => $nrVinculo,
            'NRPERIODOAPURACAO' => $nrPeriodo,
            'CDOPERADOR'        => $cdOperador,
            'DTAPURACAOSITUFREQ'=> $date,
            'NRRESUMOSITUFREQ'  => $nrResumoFreq,
            'NRSITUFREQUENCIA'  => $nrSituFreq,
            'QTREFAPURSITUFREQ' => $qtReferencia,
        ];

        $this->resumoRepository->saveRowResumoFreq($params);

        return $nrResumoFreq;
    }

    public function editRowResumoFreq(
        string $cdOperador,
        array $row
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $nrResumoFreq = (int) $row['NRRESUMOSITUFREQ'];
        $nrSituFreq = $row['NRSITUFREQUENCIA'] ?? null;
        $qtReferencia = $row['QTREFAPURSITUFREQ'] ?? null;

        if (!isset($nrSituFreq) || !isset($qtReferencia)) {
            $this->deleteRowResumoFreq($nrResumoFreq);
            return;
        }

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_CDOPERADOR'          => $cdOperador,
            'P_NRRESUMOSITUFREQ'    => $nrResumoFreq,
            'P_NRSITUFREQUENCIA'    => $nrSituFreq,
            'P_QTREFAPURSITUFREQ'   => $qtReferencia,
        ];

        $this->resumoRepository->editRowResumoFreq($params);
    }

    public function deleteRowResumoFreq(
        int $nrResumoFreq
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRRESUMOSITUFREQ'    => $nrResumoFreq,
        ];

        $this->resumoRepository->deleteRowResumoFreq($params);
    }

    public function gestorCanEditResumo() {
        $paramOrg = $this->paramOrgService->get();
        $idAlteraResumo = $paramOrg['IDALTRESUMOFREQ'];

        $canEdit = $idAlteraResumo === 'S';

        return $canEdit;
    }
}