<?php
/**
 *
 */

namespace Mig\ImportacaoXLS\Logic;

use PHPExcel\PHPExcel;

class ImportacaoXLS
{
    const ADJ_HOURS = 0.125; //Ajuste de horas por causa do GMT / Horário de Verão. Tempo em dia/hora juliano ( = hr x 1 / 24 )
    const CONST_PARAMS = ["NRORG", "NRORG_PADRAO", "COMPETENCIA"];

    protected $generalConfig;
    protected $fieldsSheet; //Dados dos campos da tabela
    protected $fieldFormats;
    protected $foreignKeys;
    protected $insertsPlanilha;
    protected $constanteParams;
    protected $linStart;

    protected $rawPlanilha;

    protected $errosDataTyping;
    protected $errosFk;
    protected $errosPk;

    protected $nomeImportacao;
    protected $errorsIgnore;
    protected $importarDados;

    protected $qtdLinhasImportadas;

    protected $entityManager;

    public function __construct(
        $jsonPath,
        $params,
        $entityManager,
        $nomeImportacao,
        $errorsIgnore,
        $importarDados
    ) {
        error_reporting(E_ERROR | E_PARSE);

        $json = file_get_contents(__DIR__ . $jsonPath);

        if (!$json) {
            throw new JsonPathException(
                str_replace("@", $jsonPath, "Caminho do JSON inválido: @")
            );
        }

        $json = json_decode($json, true);
        if (!$json) {
            throw new JsonSyntaxException(
                str_replace("@", $jsonPath, "Erro de sintaxe no JSON: @")
            );
        }

        $this->entityManager = $entityManager;
        $this->generalConfig = $json["config"];
        $this->fieldsSheet = $json["fields"];
        $this->foreignKeys = [];
        $this->nomeImportacao = $nomeImportacao;
        $this->errorsIgnore = $errorsIgnore;
        $this->importarDados = $importarDados;

        if (isset($json["FK"])) {
            foreach ($json["FK"] as $fieldName => $query) {
                $this->getParamsFromQuery($query, $sheetParams, $otherParams);
                $this->foreignKeys[$fieldName] = [
                    "query" => $query,
                    "sheetParams" => $sheetParams,
                    "otherParams" => $otherParams,
                ];
            }
        }

        if (isset($json["INSERTS"])) {
            foreach ($json["INSERTS"] as $fieldName => $query) {
                $this->getParamsFromQuery($query, $sheetParams, $otherParams);
                $this->insertsPlanilha[$fieldName] = [
                    "query" => $query,
                    "sheetParams" => $sheetParams,
                    "otherParams" => $otherParams,
                ];
            }
        }

        foreach ($params as $k => $v) {
            $this->constanteParams[$k] = $v;
        }
    }

    protected function getParamsFromQuery($query, &$sheetParams, &$otherParams)
    {
        $params = [];
        $sheetParams = [];
        $otherParams = [];

        preg_match_all("/:(\w+)/", $query, $params);

        foreach ($params[1] as $param) {
            if (isset($this->fieldsSheet[$param])) {
                $sheetParams[] = $param;
            } else {
                $otherParams[] = $param;
            }
        }
    }

    public function loadPlanilha($path)
    {
        //TODO: Verificar se ordem das colunas está correta

        $this->errosDataTyping = null;

        $this->rawPlanilha = [];

        $file = \PHPExcel_IOFactory::load($path);
        $file->setActiveSheetIndex(0);

        $headColumns = array_keys($this->fieldsSheet);
        $headPrincipal = $this->generalConfig["headPrincipal"];
        $this->linStart = isset($this->generalConfig["linStart"])
            ? $this->generalConfig["linStart"]
            : 2; //todo
        $colStart = isset($this->generalConfig["colStart"])
            ? $this->generalConfig["colStart"]
            : "A";
        $dateColumns = $this->getColumnsByType("date");

        for ($linha = $this->linStart; true; $linha++) {
            $coluna = $colStart;
            $dadosLinha = [];

            foreach ($headColumns as $h) {
                $dadosLinha[$h] = $file
                    ->getActiveSheet()
                    ->getCell($coluna++ . $linha)
                    ->getValue();
            }

            $endRows = true;

            foreach ($headPrincipal as $h) {

                if (!empty($dadosLinha[$h])) {
                    $endRows = false;
                }
            }

            if ($endRows) {
                break;
            }

            foreach ($dateColumns as $d) {
                if (
                    !is_null($dadosLinha[$d]) &&
                    gettype($dadosLinha[$d]) !== "string"
                ) {
                    $dadosLinha[$d] = date(
                        "d/m/Y",
                        \PHPExcel_Shared_Date::ExcelToPHP(
                            $dadosLinha[$d] + self::ADJ_HOURS
                        )
                    );
                }
            }

            $this->rawPlanilha[] = $dadosLinha;
        }
    }

    protected function getColumnsByType($type)
    {
        $columns = [];

        foreach ($this->fieldsSheet as $fieldName => $fieldContent) {
            if (isset($fieldContent["type"]) && $fieldContent["type"] == $type) {
                $columns[] = $fieldName;
            }
        }
        return $columns;
    }

    public function checkAllDataTyping()
    {
        $typing = new Typing();

        foreach ($this->rawPlanilha as $line => $data) {
            foreach ($data as $field => $value) {
                $typing->check($field, $value, $this->fieldsSheet, $line);
            }
        }

        $this->errosDataTyping = $typing->getErros();
    }

    public function applyFieldFormat()
    {
        $this->getFieldFormats();

        $formatter = new Formatter();

        foreach ($this->rawPlanilha as &$row) {
            foreach ($this->fieldFormats as $fieldName => $format) {
                $row[$fieldName] = $formatter->setFormat(
                    $row[$fieldName],
                    $format
                );
            }
        }
    }

    protected function getFieldFormats()
    {
        $this->fieldFormats = [];

        foreach ($this->fieldsSheet as $fieldName => $data) {
            if (isset($data["format"])) {
                $this->fieldFormats[$fieldName] = $data["format"];
            }
        }
    }

    public function checkFkValues()
    {
        $fk = new Fk(
            $this->fieldsSheet,
            $this->foreignKeys,
            $this->entityManager,
            $this->constanteParams
        );

        $fk->check($this->rawPlanilha);

        $this->errosFk = $fk->getErros();
    }

    public function insertDados(): void
    {
        $insert = new Insert(
            $this->entityManager,
            $this->rawPlanilha,
            $this->constanteParams,
            $this->insertsPlanilha,
            $this->mergedErrors(),
            $this->errorsIgnore
        );

        $insert->planilha();

        $this->qtdLinhasImportadas = $insert->getQtdLinhasImportadas();

        $this->errosPk = $insert->getErrosPk();
    }

    public function getErrosImportacao(): array
    {
        $erros = $this->mergedErrors();

        if (!empty($erros)) {
            $this->createLogErrors();
        } else {
            $log = new Log($this->entityManager, $this->logParams());

            $log->clean();
        }

        return $erros ?? [];
    }

    private function createLogErrors(): void
    {
        $file = new File(
            $this->constanteParams,
            $this->nomeImportacao,
            $this->mergedErrors(),
            $this->linStart
        );

        $log = new Log($this->entityManager, $this->logParams());

        $file->createLogFile();

        $log->set();
    }

    //TODO: Sysdate in query
    public function logParams(): array
    {
        date_default_timezone_set("America/Sao_Paulo");

        return [
            "NRORG" => $this->constanteParams["NRORG"],
            "NRMIGRACAO" => $this->constanteParams["NRMIGRACAO"],
            "IDATIVO" => "S",
            "NRORGINCLUSAO" => $this->constanteParams["NRORG"],
            "DTINCLUSAO" => date("d/m/Y H:i:s"),
            "CDOPERINCLUSAO" => $this->constanteParams["CDOPERADOR"],
            "NRORGULTATU" => $this->constanteParams["NRORG"],
            "DTULTATU" => date("d/m/Y H:i:s"),
            "CDOPERULTATU" => $this->constanteParams["CDOPERADOR"],
            "LOGPATH" =>
                "/tmp/" .
                $this->constanteParams["NRORG"] .
                "/migracao/logs" .
                "/" .
                $this->nomeImportacao .
                ".txt",
        ];
    }

    private function mergedErrors(): array
    {
        $typing = $this->getErrosDataTyping();
        //$fk = $this->getErrosFk();
        $pk = $this->getErrosPk();

        $merged = [];

        foreach ($typing as $key => $erros) {
            foreach ($erros as $erro) {
                $merged[$key][] = $erro;
            }
        }

        /*foreach ($fk as $key => $erros) {
            foreach ($erros as $erro) {
                $merged[$key][] = $erro;
            }
        }*/

        foreach ($pk as $key => $erros) {
            foreach ($erros as $erro) {
                $merged[$key][] = $erro;
            }
        }

        return $merged ?? [];
    }

    private function getErrosDataTyping(): array
    {
        return $this->errosDataTyping;
    }

    private function getErrosFk(): array
    {
        return $this->errosFk;
    }

    private function getErrosPk(): array
    {
        return $this->errosPk ?? [];
    }

    public function getQtdLinhasImportadas(): int
    {
        return $this->qtdLinhasImportadas;
    }
}

class Typing
{
    private $error;
    private $isValidType;

    public function __construct()
    {
        $this->error = new Error();
    }

    public function check($field, $valueField, $fieldsSheet, $line)
    {
        foreach ($fieldsSheet[$field] as $property => $valueProperty) {
            if ($this->validProperty($property)) {
                $this->$property(
                    $valueProperty,
                    $valueField,
                    $fieldsSheet[$field]["label"],
                    $line
                );
            }
        }
    }

    public function type($valueProperty, $valueField, $label, $line)
    {
        $this->isValidType = true;
        if ($this->checkHasNoValue($valueField)) {
            return;
        }

        if ($valueProperty == "number") {
            $valueField = str_replace(",", ".", $valueField);
            if (!is_numeric($valueField)) {
                $this->isValidType = false;
                $this->error->setErro(
                    "type",
                    $valueProperty,
                    $valueField,
                    $label,
                    $line
                );
            }
        }

        if ($valueProperty == "date" && !$this->isDate($valueField)) {
            $this->isValidType = false;
            $this->error->setErro(
                "type",
                $valueProperty,
                $valueField,
                $label,
                $line
            );
        }
    }

    public function nullable($valueProperty, $valueField, $label, $line)
    {
        if (!$valueProperty && ($valueField == "" && $valueField != 0 || is_null($valueField))) {
            $this->error->setErro(
                "nullable",
                $valueProperty,
                $valueField,
                $label,
                $line
            );
        }
    }

    public function listValues($valueProperty, $valueField, $label, $line)
    {
        if ($this->checkHasNoValue($valueField) || !$this->isValidType) {
            return;
        }

        $valueField = strtoupper($valueField);
        $valueProperty = array_map('strtoupper', $valueProperty);

        if (!in_array($valueField, $valueProperty)) {
            $this->error->setErro(
                "listValues",
                $valueProperty,
                $valueField,
                $label,
                $line
            );
        }
    }

    public function rangeValue($valueProperty, $valueField, $label, $line)
    {
        if ($this->checkHasNoValue($valueField) || !$this->isValidType) {
            return;
        }

        $valor = $valueField;
        $inicio = $valueProperty[0];
        $fim = $valueProperty[1];

        $inicio = is_null($inicio) ? $valor : $inicio;
        $fim = is_null($fim) ? $valor : $fim;

        if ($this->isDate($inicio) || $this->isDate($fim)) {
            $inicio = \DateTime::createFromFormat("d/m/Y", $inicio);
            $fim = \DateTime::createFromFormat("d/m/Y", $fim);
            $valor = \DateTime::createFromFormat("d/m/Y", $valor);
        }

        if ($inicio > $valor || $fim < $valor) {
            $this->error->setErro(
                "rangeValue",
                $valueProperty,
                $valueField,
                $label,
                $line
            );
        }
    }

    public function rangeLength($valueProperty, $valueField, $label, $line)
    {
        if ($this->checkHasNoValue($valueField) || !$this->isValidType) {
            return;
        }

        $valor = strlen($valueField);
        $inicio = $valueProperty[0];
        $fim = $valueProperty[1];

        $inicio = is_null($inicio) ? $valor : $inicio;
        $fim = is_null($fim) ? $valor : $fim;

        if ($inicio > $valor || $fim < $valor) {
            $this->error->setErro(
                "rangeLength",
                $valueProperty,
                $valueField,
                $label,
                $line
            );
        }
    }

    protected function validProperty($property)
    {
        return $property != "label" &&
            $property != "format" &&
            $property != "FK";
    }

    protected function isDate($date)
    {
        $date = \DateTime::createFromFormat("d/m/Y", $date);
        $date_errors = \DateTime::getLastErrors();

        return $date_errors["warning_count"] + $date_errors["error_count"] == 0;
    }

    protected function checkHasNoValue($value)
    {
        return is_null($value) || $value == "";
    }

    public function getErros()
    {
        return $this->error->getMessages();
    }
}

class Formatter
{
    const PAD_LENGTH = 2;
    const PAD_STRING = 0;
    const PAD_TYPE = 1;
    const PAD_TYPE_MAP = [
        "left" => STR_PAD_LEFT,
        "right" => STR_PAD_RIGHT,
        "both" => STR_PAD_BOTH,
    ];
    private const UTF8 = [
        "/[áàâãªä]/u" => "a",
        "/[ÁÀÂÃÄ]/u" => "A",
        "/[ÍÌÎÏ]/u" => "I",
        "/[íìîï]/u" => "i",
        "/[éèêë]/u" => "e",
        "/[ÉÈÊË]/u" => "E",
        "/[óòôõºö]/u" => "o",
        "/[ÓÒÔÕÖ]/u" => "O",
        "/[úùûü]/u" => "u",
        "/[ÚÙÛÜ]/u" => "U",
        "/ç/" => "c",
        "/Ç/" => "C",
        "/ñ/" => "n",
        "/Ñ/" => "N",
        "/–/" => "-", // UTF-8 hyphen to "normal" hyphen
        "/[’‘‹›]/u" => " ", // Literally a single quote
        "/[“”«»„]/u" => " ", // Double quote
        "/ /" => " ", // nonbreaking space (equiv. to 0x160)
    ];

    public function setFormat($value, $format)
    {
        foreach ($format as $property => $valueProperty) {
            $value = $this->$property($value, $valueProperty);
        }
        return $value;
    }

    protected function case($value, $valueProperty)
    {
        switch ($valueProperty) {
            case "upper":
                return mb_strtoupper($value, "UTF-8");
                break;
            case "lower":
                return mb_strtolower($value, "UTF-8");
                break;
            default:
                return $value;
        }
    }

    protected function trunc($value, $valueProperty)
    {
        return substr($value, 0, $valueProperty);
    }

    protected function removeSpecialChar($value, $valueProperty)
    {
        return $valueProperty
            ? preg_replace(
                array_keys(self::UTF8),
                array_values(self::UTF8),
                $value
            )
            : $value;
    }

    protected function pad($value, $valueProperty)
    {
        if (is_null($value) || $value == "") {
            return $value;
        }

        return str_pad(
            $value,
            $valueProperty[self::PAD_LENGTH],
            $valueProperty[self::PAD_STRING],
            self::PAD_TYPE_MAP[$valueProperty[self::PAD_TYPE]]
        );
    }

    protected function replace($value, $valueProperty)
    {
        foreach ($valueProperty as $regexs) {
            $value = preg_replace($regexs[0], $regexs[1], $value);
        }
        return $value;
    }

    protected function toFloat($value, $valueProperty)
    {
        if ($valueProperty) {
            $value = str_replace(',', '.', $value);
            return floatval($value);
        }

        return $value;
    }
}

class Fk
{
    private $fieldsSheet;
    private $foreignKeys;

    private $entityManager;
    private $error;

    private $constanteParams;

    public function __construct(
        $fieldsSheet,
        $foreignKeys,
        $entityManager,
        $constanteParams
    ) {
        $this->fieldsSheet = $fieldsSheet;
        $this->foreignKeys = $foreignKeys;
        $this->entityManager = $entityManager;
        $this->error = new Error();
        $this->constanteParams = $constanteParams;
    }

    public function check($rows)
    {
        foreach ($rows as $line => $row) {
            foreach ($this->foreignKeys as $fieldName => $fk) {
                if (is_null($row[$fieldName]) || $row[$fieldName] == "") {
                    continue;
                }

                $param = $this->populateParams(
                    $fk["sheetParams"],
                    $fk["otherParams"],
                    $row
                );

                if (!$this->existFk($fieldName, $param)) {
                    $this->error->FK($fieldName, $line, $this->fieldsSheet);
                }
            }
        }
    }

    private function populateParams($sheetParams, $otherParams, $row): array
    {
        $params = [];

        foreach ($sheetParams as $sheetParam) {
            $params[$sheetParam] = $row[$sheetParam];
        }

        foreach ($otherParams as $otherParam) {
            $params[$otherParam] = $this->constanteParams[$otherParam];
        }

        return $params;
    }

    private function existFk($fieldName, $param): bool
    {
        $query = $this->foreignKeys[$fieldName]["query"];
        $query = $this->entityManager->getConnection()->prepare($query);
        $query->execute($param);
        $result = $query->fetchAll();

        return count($result) > 0;
    }

    public function getErros()
    {
        return $this->error->getMessages();
    }
}

// TODO: Refatorar classe
class Insert
{
    private $entityManager;
    private $rawPlanilha;
    private $constanteParams;
    private $insertsPlanilha;
    private $mergedErrors;
    private $errorsIgnore;
    private $errors;
    private $qtdLinhasImportadas;
    private $tabela;

    public function __construct(
        $entityManager,
        $rawPlanilha,
        $constanteParams,
        $insertsPlanilha,
        $mergedErrors,
        $errorsIgnore
    ) {
        $this->entityManager = $entityManager;
        $this->rawPlanilha = $rawPlanilha;
        $this->constanteParams = $constanteParams;
        $this->insertsPlanilha = $insertsPlanilha;
        $this->mergedErrors = $mergedErrors;
        $this->errorsIgnore = $errorsIgnore;
    }

    public function planilha(): void
    {
        if ($this->errorsIgnore) {
            $this->unsetErrors();
        }

        $hasNoErros = empty($this->mergedErrors) || $this->errorsIgnore;

        if ($hasNoErros) {
            foreach ($this->rawPlanilha as $line => $raw) {
                $this->insert($raw, $line);
            }
        }
    }

    private function unsetErrors(): void
    {
        foreach ($this->mergedErrors as $key => $value) {
            unset($this->rawPlanilha[$key]);
        }
    }

    private function insert($raw, $line): void
    {
        try {
            $this->entityManager->beginTransaction();

            foreach ($this->insertsPlanilha as $tabela => $insertPlanilha) {
                $this->tabela = $tabela;

                $params = $this->populateParams(
                    $insertPlanilha["sheetParams"],
                    $insertPlanilha["otherParams"],
                    $raw
                );

                //TODO: Implementar insert opcional

                // START Gambiarra -- ENDERECOPARC
                $isVazioEndereco = (is_null($params["ENDERECO"]) || $params["ENDERECO"] == "") && $tabela == "ENDERECOPARC";
                
                if ($isVazioEndereco) {
                    continue;
                }
                //END Gambiarra

                // START Gambiarra -- DEP_VINCULO
                $isVazioTipoDependencia = (is_null($params["NRTIPODEPENDE"]) || $params["NRTIPODEPENDE"] == "") && $tabela == "FPA_DEPVINCULO";

                if ($isVazioTipoDependencia) {
                    continue;
                }
                //END Gambiarra

                // START Gambiarra -- FATOR_DE_RISCO
                $isVazioVinculo = (is_null($params["NRVINCULOM"]) || $params["NRVINCULOM"] == "") && $tabela == "SST_RELVINCRISCO";
                
                if ($isVazioVinculo) {
                    continue;
                }

                $isVazioEstrutura = (is_null($params["NRESTRUTURAM"]) || $params["NRESTRUTURAM"] == "") && $tabela == "SST_RELESTRUTRISCO";
                
                if ($isVazioEstrutura) {
                    continue;
                }

                $isVazioOcupacao = (is_null($params["NROCUPACAOM"]) || $params["NROCUPACAOM"] == "") && $tabela == "SST_RELOCUPARISCO";
                
                if ($isVazioOcupacao) {
                    continue;
                }
                //END Gambiarra

                $query = $this->entityManager
                    ->getConnection()
                    ->prepare($insertPlanilha["query"]);

                $query->execute($params);
            }

            $this->entityManager->flush();
            $this->entityManager->commit();

            $this->qtdLinhasImportadas++;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            
            $chaveViolada = strpos($e->getMessage(), "ORA-00001") !== false;
            if ($chaveViolada) {
                $this->errors[$line][] = "Registro já existente na tabela: ".$this->tabela;
            } else {
                throw $e;
            }
        }
    }

    private function populateParams($sheetParams, $otherParams, $raw): array
    {
        $params = [];

        foreach ($sheetParams as $sheetParam) {
            $params[$sheetParam] = $raw[$sheetParam];
        }

        foreach ($otherParams as $otherParam) {
            $params[$otherParam] = $this->constanteParams[$otherParam];
        }

        return $params;
    }

    public function getQtdLinhasImportadas(): int
    {
        return $this->qtdLinhasImportadas ?? 0;
    }

    public function getErrosPk()
    {
        return $this->errors;
    }
}

class Error
{
    const ERROR_TYPE = "O valor do campo '@label' deve ser @valueProperty, recebido '@valueField'.";
    const ERROR_NULLABLE = "É necessário um valor para o campo '@label'.";
    const ERROR_LISTVALUES = "Valor '@valueField' do campo '@label' é inválido. Somente valores: @valueProperty, são permitidos.";
    const ERROR_RANGEVALUE = [
        "between" =>
            "O valor '@valueField' do campo '@label' deve estar entre '@0' e '@1'.",
        "onlyStart" =>
            "O valor '@valueField' do campo '@label' deve ser maior ou igual a '@0'.",
        "onlyEnd" =>
            "O valor '@valueField' do campo '@label' deve ser menor ou igual a '@1'.",
    ];
    const ERROR_RANGELENGTH = [
        "between" =>
            "O valor '@valueField' do campo '@label' deve ter um comprimento entre '@0' e '@1'.",
        "onlyStart" =>
            "O valor '@valueField' do campo '@label' deve ter um comprimento maior ou igual a '@0'.",
        "onlyEnd" =>
            "O valor '@valueField' do campo '@label' deve ter um comprimento menor ou igual a '@1'.",
    ];

    const ERROR_FK = "O valor do campo '@label@' é inválido.";

    private $erros = [];

    public function setErro(
        $property,
        $valueProperty,
        $valueField,
        $label,
        $line
    ) {
        $this->$property($valueProperty, $valueField, $label, $line);
    }

    public function type($valueProperty, $valueField, $label, $line)
    {
        $valueProperty =
            $valueProperty == "number" ? "numérico" : $valueProperty;
        $valueProperty = $valueProperty == "date" ? "uma data" : $valueProperty;

        $msg = str_replace("@valueProperty", $valueProperty, self::ERROR_TYPE);
        $msg = str_replace("@valueField", $valueField, $msg);
        $msg = str_replace("@label", $label, $msg);

        $this->erros[$line][] = $msg;
    }

    public function nullable($valueProperty, $valueField, $label, $line)
    {
        $this->erros[$line][] = str_replace(
            "@label",
            $label,
            self::ERROR_NULLABLE
        );
    }

    public function listValues($valueProperty, $valueField, $label, $line)
    {
        $ultimo =
            count($valueProperty) > 1
                ? " e '" . array_pop($valueProperty) . "'"
                : null;
        $valueProperty = "'" . join("', '", $valueProperty) . "'";
        $valueProperty .= $ultimo;

        $msg = str_replace(
            "@valueProperty",
            $valueProperty,
            self::ERROR_LISTVALUES
        );
        $msg = str_replace("@valueField", $valueField, $msg);
        $msg = str_replace("@label", $label, $msg);

        $this->erros[$line][] = $msg;
    }

    public function rangeValue($valueProperty, $valueField, $label, $line)
    {
        if (!is_null($valueProperty[0]) && !is_null($valueProperty[1])) {
            $msg = self::ERROR_RANGEVALUE["between"];
        } elseif (!is_null($valueProperty[0])) {
            $msg = self::ERROR_RANGEVALUE["onlyStart"];
        } else {
            $msg = self::ERROR_RANGEVALUE["onlyEnd"];
        }

        $msg = str_replace("@0", $valueProperty[0], $msg);
        $msg = str_replace("@1", $valueProperty[1], $msg);
        $msg = str_replace("@valueField", $valueField, $msg);
        $msg = str_replace("@label", $label, $msg);

        $this->erros[$line][] = $msg;
    }

    public function rangeLength($valueProperty, $valueField, $label, $line)
    {
        if (!is_null($valueProperty[0]) && !is_null($valueProperty[1])) {
            $msg = self::ERROR_RANGELENGTH["between"];
        } elseif (!is_null($valueProperty[0])) {
            $msg = self::ERROR_RANGELENGTH["onlyStart"];
        } else {
            $msg = self::ERROR_RANGELENGTH["onlyEnd"];
        }

        $msg = str_replace("@0", $valueProperty[0], $msg);
        $msg = str_replace("@1", $valueProperty[1], $msg);
        $msg = str_replace("@valueField", $valueField, $msg);
        $msg = str_replace("@label", $label, $msg);

        $this->erros[$line][] = $msg;
    }

    public function FK($field, $line, $fieldsSheet)
    {
        $field = $fieldsSheet[$field]["label"];

        $msg = str_replace("@label@", $field, self::ERROR_FK);

        $this->erros[$line][] = $msg;
    }

    public function getMessages()
    {
        return $this->erros;
    }
}

class File
{
    private $logsPath;
    private $nomeImportacao;
    private $errors;
    private $linStart;

    public function __construct($constanteParams, $nomeImportacao, $errors, $linStart)
    {
        $this->logsPath =
            "/tmp/" . $constanteParams["NRORG"] . "/migracao/logs";
        $this->nomeImportacao = $nomeImportacao;
        $this->errors = $errors;
        $this->linStart = $linStart;
    }

    public function createLogFile()
    {
        $file = $this->logsPath . "/" . $this->nomeImportacao . ".txt";

        $this->createFolder($this->logsPath);
        $this->writeFile($file, $this->errors);
    }

    private function createFolder($path): void
    {
        if (!file_exists($path)) {
            mkdir($path, 0777, true);
        }
    }

    private function writeFile($file, $errosArray): void
    {
        $file = fopen($file, "w");

        foreach ($errosArray as $line => $erros) {
            $line = $line + $this->linStart;

            if (is_array($erros)) {
                fwrite($file, "Linha $line:\n");

                foreach ($erros as $erro) {
                    fwrite($file, "\t" . $erro . "\n");
                }
            } else {
                fwrite($file, "Linha $line: \n\t$erros\n");
            }
        }

        fclose($file);
    }
}

class Log
{
    private $entityManager;
    private $params;

    public function __construct($entityManager, $params)
    {
        $this->entityManager = $entityManager;
        $this->params = $params;
    }

    public function set()
    {
        if ($this->hasLog()) {
            $this->update();
        } else {
            $this->insert();
        }
    }

    private function hasLog()
    {
        $query = 'SELECT 1 
                    FROM GPE_LOGMIGRACAO
                   WHERE NRORG = :NRORG 
                     AND NRMIGRACAO = :NRMIGRACAO';

        $sql = $this->entityManager->getConnection()->prepare($query);

        $binds = [
            "NRORG" => $this->params["NRORG"],
            "NRMIGRACAO" => $this->params["NRMIGRACAO"],
        ];

        $sql->execute($binds);

        return $sql->fetchAll();
    }

    private function update()
    {
        $query = 'UPDATE GPE_LOGMIGRACAO
                     SET DSMENSAGEM = :DSMENSAGEM, LOGPATH = :LOGPATH, DTULTATU = :DTULTATU
                   WHERE NRORG = :NRORG AND NRMIGRACAO = :NRMIGRACAO';

        $binds = [
            "DSMENSAGEM" => "Erro na importação",
            "LOGPATH" => $this->params["LOGPATH"],
            "DTULTATU" => $this->params["DTULTATU"],
            "NRORG" => $this->params["NRORG"],
            "NRMIGRACAO" => $this->params["NRMIGRACAO"],
        ];

        $sql = $this->entityManager->getConnection()->prepare($query);

        $sql->execute($binds);
    }

    private function insert()
    {
        $query = 'INSERT INTO GPE_LOGMIGRACAO
                     (NRORG, NRLOGMIGRACAO, NRMIGRACAO, DSMENSAGEM, IDATIVO, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, DTULTATU, CDOPERULTATU, LOGPATH)
                   VALUES
                     (:NRORG, :NRLOGMIGRACAO, :NRMIGRACAO, :DSMENSAGEM, :IDATIVO, :NRORGINCLUSAO, :DTINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :DTULTATU, :CDOPERULTATU, :LOGPATH)';

        $binds = [
            "NRORG" => $this->params["NRORG"],
            "NRLOGMIGRACAO" => $this->lastNrLogMigracao() + 1,
            "NRMIGRACAO" => $this->params["NRMIGRACAO"],
            "DSMENSAGEM" => "Erro ao importar",
            "IDATIVO" => "S",
            "NRORGINCLUSAO" => $this->params["NRORG"],
            "DTINCLUSAO" => $this->params["DTINCLUSAO"],
            "CDOPERINCLUSAO" => $this->params["CDOPERINCLUSAO"],
            "NRORGULTATU" => $this->params["NRORG"],
            "DTULTATU" => $this->params["DTULTATU"],
            "CDOPERULTATU" => $this->params["CDOPERULTATU"],
            "LOGPATH" => $this->params["LOGPATH"],
        ];

        $sql = $this->entityManager->getConnection()->prepare($query);

        $sql->execute($binds);
    }

    private function lastNrLogMigracao()
    {
        $query = 'SELECT NVL(MAX(NRLOGMIGRACAO), 0) AS NRLOGMIGRACAO
                    FROM GPE_LOGMIGRACAO
                   WHERE NRORG = :NRORG';

        $sql = $this->entityManager->getConnection()->prepare($query);

        $binds = [
            "NRORG" => $this->params["NRORG"],
        ];

        $sql->execute($binds);

        return $sql->fetchAll()[0]["NRLOGMIGRACAO"];
    }

    public function clean()
    {
        $query = 'DELETE FROM GPE_LOGMIGRACAO
                   WHERE NRORG = :NRORG AND NRMIGRACAO = :NRMIGRACAO';

        $binds = [
            "NRORG" => $this->params["NRORG"],
            "NRMIGRACAO" => $this->params["NRMIGRACAO"],
        ];

        $sql = $this->entityManager->getConnection()->prepare($query);

        $sql->execute($binds);
    }
}
class JsonPathException extends \Exception
{
    public function __construct(
        $message = "Caminho do JSON inválido.",
        $code = 0,
        \Exception $previous = null
    ) {
        parent::__construct($message, $code, $previous);
    }
}
class JsonSyntaxException extends \Exception
{
    public function __construct(
        $message = "Erro de sintaxe no JSON.",
        $code = 0,
        \Exception $previous = null
    ) {
        parent::__construct($message, $code, $previous);
    }
}

?>
